class Vectorize(object):
    def __init__ (self,DataFrame):
        self.DataFrame = DataFrame
    def Vectorize(num_vectors, vectors_keys):
        #vectors_keys = [['col1', 'col2'],[col1]]
        vectors_list = []
        for i in range(num_vectors):
            vector = []
        for row in self.DataFrame.DataFrame:
            vector.append()
